local major = "LibCandyBar-3.0"
local minor = tonumber(("$Rev: 2 $"):match("(%d+)")) or 1
if not LibStub then error("LibCandyBar-3.0 requires LibStub.") end
local cbh = LibStub:GetLibrary("CallbackHandler-1.0")
if not cbh then error("LibCandyBar-3.0 requires CallbackHandler-1.0") end
local lib, old = LibStub:NewLibrary(major, minor)
if not lib then return end
lib.callbacks = lib.callbacks or cbh:New(lib)
local cb = lib.callbacks
-- ninjaed from LibBars-1.0
lib.dummyFrame = lib.dummyFrame or CreateFrame("Frame")
lib.barFrameMT = lib.barFrameMT or {__index = lib.dummyFrame}
lib.barPrototype = lib.barProtoType or setmetatable({}, lib.barFrameMT)
lib.barPrototype_mt = lib.barPrototype_mt or {__index = lib.barPrototype}
lib.availableBars = lib.availableBars or {}

local bar = {}
local barPrototype = lib.barPrototype
local barPrototype_meta = lib.barPrototype_mt
local availableBars = lib.availableBars
local SecondsToTimeAbbrev = SecondsToTimeAbbrev
local GetTime = GetTime

local function onUpdate(self)
	local t = GetTime()
	if t >= self.exp then
		self:Stop()
	else
		local time = self.exp - t
		self.remaining = time
		self.candyBarBar:SetValue(time)
		self.candyBarDuration:SetFormattedText(SecondsToTimeAbbrev(time))
	end
end

--------------------------------------------------------------------------------
-- Bar functions
--

local function restyleBar(self)
	if not self.running then return end
	self:SetWidth(self.width)
	self:SetHeight(self.height)
	if self.candyBarIconFrame:GetTexture() then
		self.candyBarBar:SetPoint("TOPLEFT", self.candyBarIconFrame, "TOPRIGHT")
		self.candyBarBar:SetPoint("BOTTOMLEFT", self.candyBarIconFrame, "BOTTOMRIGHT")
		self.candyBarIconFrame:SetWidth(self.height)
		self.candyBarIconFrame:Show()
	else
		self.candyBarBar:SetPoint("TOPLEFT", self)
		self.candyBarBar:SetPoint("BOTTOMLEFT", self)
		self.candyBarIconFrame:Hide()
	end
	if self.candyBarLabel:GetText() then self.candyBarLabel:Show()
	else self.candyBarLabel:Hide() end
	if self.showTime then self.candyBarDuration:Show()
	else self.candyBarDuration:Hide() end
end

function barPrototype:SetColor(r, g, b, a) self.candyBarBar:SetStatusBarColor(r, g, b, a) end
function barPrototype:SetLabel(text) self.candyBarLabel:SetText(text); restyleBar(self) end
function barPrototype:SetIcon(icon) self.candyBarIconFrame:SetTexture(icon); restyleBar(self) end
function barPrototype:SetTimeVisibility(bool) self.showTime = bool; restyleBar(self) end
function barPrototype:SetDuration(duration) self.remaining = duration end
function barPrototype:Start()
	self.running = true
	restyleBar(self)
	self.candyBarBar:SetMinMaxValues(0, self.remaining)
	self.exp = GetTime() + self.remaining
	self:SetScript("OnUpdate", onUpdate)
	self:Show()
end
function barPrototype:Stop()
	self.running = nil
	self:SetScript("OnUpdate", nil)
	availableBars[self] = true
	self:Hide()
	cb:Fire("LibCandyBar_Stop", self)
end

--------------------------------------------------------------------------------
-- Library functions
--

function lib:New(texture, width, height)
	local bar = next(availableBars)
	if not bar then
		local frame = CreateFrame("Frame")
		bar = setmetatable(frame, barPrototype_meta)
		bar:SetScale(1)
		bar:SetMovable(1)
		local icon = bar:CreateTexture(nil, "BACKGROUND")
		icon:SetPoint("TOPLEFT", bar)
		icon:SetPoint("BOTTOMLEFT", bar)
		icon:SetTexCoord(0.07, 0.93, 0.07, 0.93)
		bar.candyBarIconFrame = icon
		local statusbar = CreateFrame("StatusBar", nil, bar)
		statusbar:SetPoint("TOPRIGHT", bar)
		statusbar:SetPoint("BOTTOMRIGHT", bar)
		bar.candyBarBar = statusbar
		local bg = statusbar:CreateTexture(nil, "BACKGROUND")
		bg:SetAllPoints()
		bg:SetVertexColor(0.5, 0.5, 0.5, 0.3)
		bar.candyBarBackground = bg
		local duration = statusbar:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmallOutline")
		duration:SetPoint("RIGHT", statusbar, -2, 0)
		bar.candyBarDuration = duration
		local name = statusbar:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmallOutline")
		name:SetAllPoints(bar)
		name:SetJustifyH("CENTER")
		name:SetJustifyV("MIDDLE")
		bar.candyBarLabel = name
	else
		bar:SetLabel(nil)
		bar:SetIcon(nil)
		bar:SetDuration(nil)
		availableBars[bar] = nil
	end
	bar.candyBarBar:SetStatusBarTexture(texture)
	bar.candyBarBackground:SetTexture(texture)
	bar.width = width
	bar.height = height
	bar:Hide()
	return bar
end

